/*
	CarTheft v1.0 By [NL]Mox 2008
	
	This is my first SA:MP gamemode. Enjoy! Please send bugreports to samp@mox.designt.nl
	
	Cartheft is a team-based CTF type gameplay.
	Each team has a garage filled with cars they cannot enter.
	The other team can enter these cars and steal them by driving them to a checkpoint.
	Money and points are awarded for car-steals (depending on the cars health) or defensive actions.
	Weapons or vehicles can be bought using the /buy command.

	Known bugs:
	- Server crashes once in a while. (no clue why, segfault probably SA:MP bug)
	- Clients crash sometimes.
	- /rcon gmx will crash the server (textdraws). use /restart instead.
	
    Copyright (C) 2008 Jesper Jeeninga

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
	= Credits =
	Thanks to Joost for testing and feedback.
	Thanks to the SA:MP team for their great project and WIKI
*/
#include <a_samp>
#include <float>
#include <core>

//coordinates
#define WORLD_BOUNDS -1485.7008, -2339.9888, 1403.3059, -234.1424 // (x_max,x_min,y_max,y_min)
#define CHECK_TEAM1 -2026.8754,125.3854,28.8051
#define CHECK_TEAM2 -1500.2572,1273.8842,6.7793
#define ICON_TEAM1 -1957.0669,270.8234,60
#define ICON_TEAM2 -1663.7291,1221.1833,60
#define HOMEAREA_TEAM1 -2085.668, -1811.417, 175.8956, 440.2082 // (x_min,x_max,y_min,y_max)
#define HOMEAREA_TEAM2 -1803.692, -1525.579, 1138.467, 1390.945 // (x_min,x_max,y_min,y_max)

//money amounts
#define AWARD_HOMEDEFENCE 10000
#define AWARD_CARDEFENCE 25000
#define PENALTY_SUICIDE 5000
#define PENALTY_DEATH 900

//point amounts
#define SCORE_WINLIMIT 5000
#define SCORE_STEAL_CAR 175
#define SCORE_DEFEND_CAR 100
#define SCORE_DEFEND_BASE 100
#define SCORE_KILL 5
#define SCORE_OTHERS_WRECKED_CAR 25

//other
#define TEAM_NEUTRAL 0
#define TEAM_1 1
#define TEAM_2 2

#define TYPE_WEAPON 0
#define TYPE_VEHICLE 1
#define TYPE_OBJECT 2

#define COLOR_GREY 0xAFAFAFAA
#define COLOR_GREEN 0x33AA33AA
#define COLOR_PURPLE 0xA86EFCAA
#define COLOR_ORANGE 0xFF8000FF
#define COLOR_LIGHTBLUE 0x00BFFFFF
#define COLOR_YELLOW 0xFFD700AA

//sounds
#define SOUND_WIN 1183
#define SOUND_LOOSE 1142
/*
#define SOUND_CROWD_AWWS 36200
#define SOUND_CROWD_AWWS2 36201
#define SOUND_CROWD_CHEERS 36202
#define SOUND_CROWD_CHEERS2 36203
#define SOUND_CROWD_CHEERS3 36204
#define SOUND_CROWD_CHEERS_BIG 36205
*/

//anticheat
#define RESTRICTED_WEAPONS WEAPON_CHAINSAW,WEAPON_SHOTGUN,WEAPON_SAWEDOFF,WEAPON_SHOTGSPA,WEAPON_UZI,WEAPON_AK47,WEAPON_M4,WEAPON_TEC9,WEAPON_RIFLE,WEAPON_SNIPER,WEAPON_MINIGUN,WEAPON_SATCHEL
#define ANTICHEAT_MAX_MONEY 500000

forward TeamBalanceCheck();
forward EndGame();
forward ExitGameMode();

new i; //for all loops;
new gStaticVehicles; // Any ID less than this one is allowed to respawn;
new Menu:buymenu; // Buy menu
new Text:gScore; //Top Score text
new gRespawnDelay = 120; // Respawn delay for cars
new gTeamWon = 0;

enum TeamInfo {
	TeamName[20],
	TeamColor,
	TeamCarColor,
	TeamScoreName[23], // 3 chars color + exactly 20 for team (space filled)
	TeamScore, 
	TeamIcon
}

new gTeamInfo[][TeamInfo]  =
{
	{ "Neutral", COLOR_GREY, 0, 0, 1},
	{ "Groove Street", COLOR_GREEN, 86	,"~g~Groove Street", 0, 62 },
	{ "Ballas", COLOR_PURPLE, 5 		,"~p~Ballas       ", 0, 59 }
};

// What team and what skin is used
enum PlayerInfo {
	Team,
	Skin,
	Sound
}
new gPlayer[MAX_PLAYERS][PlayerInfo];

enum CarSpawnInfo {
	CarTypeId,
	Float:CarX,
	Float:CarY,
	Float:CarZ,
	Float:CarAngle,
	CarName[10],
	CarTeam,
	CarCostPrice,
	CarSellPrice,
	StolenByPlayerId
}

new gCars[][CarSpawnInfo] =
{
	//Dummy Car to make first car start with id = 1
	{587, -1797.0400, 1312.5695, 59.4626, 356.7061, "Dummy"   , TEAM_NEUTRAL, 0,0, -1},
	//Ballas Cars
	{495, -1661.3176, 1213.6340, 6.9348 , 285.4558, "Sandking", TEAM_2, 5000, 15000, -1},
	{555, -1647.8245, 1208.2153, 13.3575, 229.2822, "Windsor" , TEAM_2, 10000, 20000, -1},
	{477, -1660.6287, 1217.6010, 12.3562, 271.6462, "ZR-350"  , TEAM_2, 15000, 25000, -1},
	{451, -1659.4178, 1214.8815, 20.8405, 280.4525, "Turismo", TEAM_2, 15000, 25000, -1},
	{480, -1663.7291, 1221.1833, 20.8400, 277.8721, "Comet"   , TEAM_2, 15000, 25000, -1},
	{535, -1647.4937, 1208.3745, 20.8385, 224.2401, "Slamvan" , TEAM_2, 30000, 35000, -1},
	//Grove Street Cars
	{495, -1956.6973, 258.6066, 34.1534, 89.0696 , "Sandking", TEAM_1, 5000, 15000, -1},
	{555, -1958.3944, 299.3045, 34.1549, 89.5054 , "Windsor" , TEAM_1, 10000, 20000, -1},
	{477, -1958.3500, 283.1705, 34.1529, 91.3167 , "ZR-350"  , TEAM_1, 15000, 25000, -1},
	{451, -1947.6989, 262.5674, 40.7311, 214.0426, "Turismo", TEAM_1, 15000, 25000, -1},
	{480, -1957.0669, 270.8234, 34.1525, 88.3038 , "Comet"   , TEAM_1, 15000, 25000, -1},
	{535, -1955.6499, 303.5610, 40.7315, 179.3499, "Slamvan" , TEAM_1, 30000, 35000, -1}
};

enum PlayerSpawnInfo {
	Float:PlayerX,
	Float:PlayerY,
	Float:PlayerZ,
	Float:PlayerAngle,
}

new gTeamGreenSpawns[][PlayerSpawnInfo] =
{
	{-1952.4290, 305.8105, 35.4688, 145.8504},
	{-1951.7891, 280.1732, 35.4688, 59.9963},
	{-1942.6361, 276.9001, 35.4739, 138.9570},
	{-1942.7406, 276.2732, 41.0471, 112.3000},
	{-1957.3210, 254.2536, 41.0471, 343.7854}
};

new gTeamPurpleSpawns[][PlayerSpawnInfo] =
{
	{-1683.4409, 1211.0834, 21.1563, 269.6597},
	{-1665.2560, 1228.2209, 21.1563, 185.9990},
	{-1683.4838, 1210.0364, 13.6719, 271.2031},
	{-1670.0055, 1203.5775, 7.2546, 306.6101}
};

enum BuyMenuInfo {
	Name[24],
	Type,
	PriceString[16],
	Price,
	Id,
	Ammo
}

new gBuyMenuItems[][BuyMenuInfo] =
{
	{ "Explosive Barrel",TYPE_OBJECT	,"1.000 $"	, 1000	,1225,0},
	{ "Pistol" 			,TYPE_WEAPON	,"1.500 $"	, 1500	,WEAPON_DEAGLE ,42},
	{ "Rocket Launcher"	,TYPE_WEAPON	,"5.000 $"	, 5000	,WEAPON_ROCKETLAUNCHER ,5},
	{ "Mp5"				,TYPE_WEAPON	,"5.000 $"	, 5000	,WEAPON_MP5 ,90},
	{ "Bike"   			,TYPE_VEHICLE	,"5.000 $"	, 5000	,581,0},
	{ "Taxi"   			,TYPE_VEHICLE	,"15.000 $"	, 15000	,420,0},
//	{ "Minivan"			,TYPE_VEHICLE	,"15.000 $"	, 15000	,418,0},
	{ "Super GT"   		,TYPE_VEHICLE	,"20.000 $"	, 20000	,506,0},
	{ "Dozer"   		,TYPE_VEHICLE	,"40.000 $"	, 40000	,486,0},
	{ "FBI Truck"   	,TYPE_VEHICLE	,"100.000 $",100000	,528,0}
};

main()
{
	print("\n----------------------------------");
	print(" Car Theft version 1.0 by [NL]Mox");
	print("----------------------------------\n");
}

public OnGameModeInit()
{
	printf("Initialising...");
	SetGameModeText("Car Theft");
	SetWorldTime(4);
	SetTeamCount(3);
	EnableTirePopping(1);
	EnableStuntBonusForAll(0);
	DisableInteriorEnterExits();
	SetTimer("TeamBalanceCheck",60000,1); //check every minute
	gTeamWon = 0;
//Create the team score
	new string[64];
	format(string, 64, "%s:~w~ 0 ~n~%s:~w~ 0", gTeamInfo[TEAM_2][TeamScoreName], gTeamInfo[TEAM_1][TeamScoreName]);
	gScore = TextDrawCreate(floatadd(320.0, (sizeof(string) / 2)), 0, string );
	TextDrawAlignment(gScore, 3);
	TextDrawFont(gScore, 2);
	TextDrawSetShadow(gScore, 0);
	TextDrawSetOutline(gScore, 1);
//Create the buymenu
	buymenu = CreateMenu("Buy Menu", 2, 200.0, 100.0, 150.0, 150.0);
	SetMenuColumnHeader(buymenu,0, "Item");
	SetMenuColumnHeader(buymenu,1, "Price");
	for(i=0;i<sizeof(gBuyMenuItems);i++) {
		AddMenuItem(buymenu, 0, gBuyMenuItems[i][Name]);
		AddMenuItem(buymenu, 1, gBuyMenuItems[i][PriceString]);
	}
// Ballas Players (Purple)
    AddPlayerClassEx(TEAM_2, 102, 0.0, 0.0, 0.0, 0.0, 0, 0, 0, 0, 0, 0);
    AddPlayerClassEx(TEAM_2, 103, 0.0, 0.0, 0.0, 0.0, 0, 0, 0, 0, 0, 0);
    AddPlayerClassEx(TEAM_2, 104, 0.0, 0.0, 0.0, 0.0, 0, 0, 0, 0, 0, 0);
// Grove Street Players (Green)
	AddPlayerClassEx(TEAM_1, 105, 0.0, 0.0, 0.0, 0.0, 0, 0, 0, 0, 0, 0);
	AddPlayerClassEx(TEAM_1, 106, 0.0, 0.0, 0.0, 0.0, 0, 0, 0, 0, 0, 0);
	AddPlayerClassEx(TEAM_1, 107, 0.0, 0.0, 0.0, 0.0, 0, 0, 0, 0, 0, 0);
	for(i=0;i<sizeof(gPlayer);i++) {
	    gPlayer[i][Sound] = -1;
   	    gPlayer[i][Team] = TEAM_NEUTRAL;
   	    gPlayer[i][Skin] = 0;
	}
// Team Cars
	for (i=1;i<sizeof(gCars);i++) {
		AddStaticVehicleEx(
			gCars[i][CarTypeId],
			gCars[i][CarX],
			gCars[i][CarY],
			gCars[i][CarZ],
			gCars[i][CarAngle],
			gTeamInfo[gCars[i][CarTeam]][TeamCarColor],
			1,
			gRespawnDelay);
		//printf("[Init] Car %d - %s,team %s, cost: %d", i, gCars[i][CarName], gTeamInfo[gCars[i][CarTeam]][TeamName], gCars[i][CarCostPrice]);
	}
// Green Area
	//Fences to block the back of the team1 area
	//front ramp
	CreateObject(971, -1967.08, 246.684, 38, 0, 0, 270);
	CreateObject(971, -1967.08, 243.684, 38, 0, 0, 90);
	//back ramp
	CreateObject(969, -1905.37, 238.339, 34.5078, 0, 0, 180);
	CreateObject(969, -1910.9,  238.339, 34.5078, 0,0,0);
	//back court
	CreateObject(971, -1920.6, 277.502, 43.65, 0, 0, 0);
	CreateObject(971, -1916.15, 273.03, 43.65, 0, 0, 90);
	CreateObject(971, -1916.15, 264.15, 43.65, 0, 0, 90);
	CreateObject(971, -1916.15, 255.28, 43.65, 0, 0, 90);
	//ramp to roof
	CreateObject(3037, -1921.13, 279.825, 43.65, 45, 90, 90);
// Vehicles
	AddStaticVehicle(481,-1980.7898,236.0476,29.4965,89.2364,-1, gTeamInfo[TEAM_1][TeamCarColor]); // Bike left of garage
	AddStaticVehicle(468,-1987.2245,131.2239,27.3366,177.6530,-1,gTeamInfo[TEAM_1][TeamCarColor]); // Sanchez
	AddStaticVehicle(462,-1981.7852,308.5378,34.7715,117.6091,-1,gTeamInfo[TEAM_1][TeamCarColor]); // vespa
	AddStaticVehicle(462,-1978.9185,307.8849,34.7714,146.9252,-1,gTeamInfo[TEAM_1][TeamCarColor]); // vespa
	AddStaticVehicle(462,-1975.6742,307.8518,34.7713,140.8150,-1,gTeamInfo[TEAM_1][TeamCarColor]); // vespa
// Weapons
	AddStaticPickup(359,2,-1907.6265,298.6503,40.6442); //Rocketlauncher
	AddStaticPickup(359,2,-2068.6079,308.2984,41.9922); //Rocketlauncher
	AddStaticPickup(348,2,-2124.0728,220.1626,35.2415); //Desert Eagle
	AddStaticPickup(348,2,-2052.3486,303.4186,41.9922); //Desert Eagle
// Purple Area
	//Jump
	CreateObject(1245, -1631.02, 1171.04, 24, 0, 0, 290);
// Vehicles
	AddStaticVehicle(462,-1693.2654,1254.5167,6.8177,190.0824,-1,gTeamInfo[TEAM_2][TeamCarColor]); // vespa
	AddStaticVehicle(462,-1696.1554,1257.1904,6.8156,181.3308,-1,gTeamInfo[TEAM_2][TeamCarColor]); // vespa
	AddStaticVehicle(462,-1699.5995,1260.7712,6.8179,184.6544,-1,gTeamInfo[TEAM_2][TeamCarColor]); // vespa
	AddStaticVehicle(468,-1695.5575,1337.9542,6.7792,130.7655,-1,gTeamInfo[TEAM_2][TeamCarColor]); // Sanchez
	//Parking
	AddStaticVehicle(418,-1784.8840,1311.3420,31.5798,358.4093,-1,gTeamInfo[TEAM_2][TeamCarColor]); // Moonbeam (parking)
	AddStaticVehicle(414,-1811.4478,1294.3934,31.5790,6.3963,-1,gTeamInfo[TEAM_2][TeamCarColor]); // Mule (parking)
	AddStaticVehicle(543,-1817.2760,1310.9263,40.8771,8.8663,-1,gTeamInfo[TEAM_2][TeamCarColor]); // Sadler (parking)
	AddStaticVehicle(589,-1810.4607,1293.9431,50.1734,180.8750,-1,gTeamInfo[TEAM_2][TeamCarColor]); // Club (parking)
	AddStaticVehicle(434,-1833.6331,1308.5907,59.4600,344.2856,-1,gTeamInfo[TEAM_2][TeamCarColor]); // Hot Knife (top parking)
	AddStaticVehicle(576,-1797.0400,1312.5695,59.4626,356.7061,-1,gTeamInfo[TEAM_2][TeamCarColor]); // Tornado (top parking)
// Weapons
	AddStaticPickup(359,2,-1644.3014,1202.1437,32.9332); //Rocketlauncher
	AddStaticPickup(359,2,-1721.0330,1275.9446,17.9219); //Rocketlauncher
	AddStaticPickup(348, 2,-1573.9111,1267.4441,1.4531); //Desert Eagle
	AddStaticPickup(348, 2,-1796.0397,1228.8236,32.6487); //Desert Eagle
// Neutral Area
    AddStaticVehicle(509,-1965.9290,510.8040,34.7693,88.3696,-1,-1); // Bike
	AddStaticVehicle(448,-1825.9170,616.4012,34.7688,302.7365,-1,-1); // Pizza
    AddStaticVehicle(448,-1805.0955,950.9558,24.4876,178.1594,-1,-1); // Pizza
    AddStaticVehicle(471,-1786.6709,1205.1561,24.8523,179.2491,-1,-1); // Quad
	AddStaticVehicle(414,-1984.7643,950.6115,45.1748,233.5343,-1,-1); // Mule
	AddStaticVehicle(589,-1996.2708,683.1037,45.0946,182.0916,-1,-1); // Club
	AddStaticVehicle(440,-2103.2012,657.8365,52.1256,87.5354 ,-1,-1); // Rumpo
	AddStaticVehicle(571,-1978.4960,432.1438,25.2523,0.5046,-1,-1); // Kart (under road)
	AddStaticVehicle(499,-1932.4310,584.5154,34.8596,179.3360,-1,-1); // Benson
	AddStaticVehicle(609,-1762.2977,948.6586,24.4691,268.1806,-1,-1); // BoxBurg
	AddStaticVehicle(400,-1703.7616,1007.6793,17.3105,89.1287,-1,-1); // Landstalker
	AddStaticVehicle(505,-1694.8922,1058.1055,17.3123,0.6419,-1,-1); // Rancher
// Weapons
	AddStaticPickup(348, 2, -1950.3108, 797.9608, 55.7230); //Desert Eagle
	AddStaticPickup(348, 2, -1697.1071,1011.9481, 44.8066); //Desert Eagle
	AddStaticPickup(353, 2, -2066.9495,833.8677,71.8906); //MP5
	AddStaticPickup(353, 2, -2017.4365,879.9490,62.7734); //MP5
	
// This should be at the last of all static vehicle declarations, vehicles defined after this won't respawn.
	gStaticVehicles = AddStaticVehicle(544,-2021.6923,93.4554,27.7743,99.8379,-1,-1); // Firetruck

//Non-respawn Vehicles (just for the first round)
	//team1
	AddStaticVehicle(479,-2032.9877,303.9760,34.9369,91.8372,-1,gTeamInfo[TEAM_1][TeamCarColor]);
	AddStaticVehicle(479,-1985.8340,271.0069,34.8838,84.1060,-1,gTeamInfo[TEAM_1][TeamCarColor]);
	AddStaticVehicle(479,-1986.4967,264.7715,34.8831,86.4116,-1,gTeamInfo[TEAM_1][TeamCarColor]);
	AddStaticVehicle(479,-1987.9084,258.4109,34.8784,83.3878,-1,gTeamInfo[TEAM_1][TeamCarColor]);
	AddStaticVehicle(479,-1988.2712,247.0588,34.8764,84.0795,-1,gTeamInfo[TEAM_1][TeamCarColor]);
	//team2
	AddStaticVehicle(479,-1584.1571,1228.5947,6.8920,313.8981,-1,gTeamInfo[TEAM_2][TeamCarColor]);
	AddStaticVehicle(479,-1577.4921,1228.3004,6.8933,320.8593,-1,gTeamInfo[TEAM_2][TeamCarColor]);
	AddStaticVehicle(479,-1566.5537,1228.9033,6.8918,323.1938,-1,gTeamInfo[TEAM_2][TeamCarColor]);
	AddStaticVehicle(479,-1665.0494,1236.9784,6.8136,40.7568,-1,gTeamInfo[TEAM_2][TeamCarColor]);
	AddStaticVehicle(479,-1621.9540,1170.3063,6.9446,221.8188,-1,gTeamInfo[TEAM_2][TeamCarColor]);
	printf("Initialisation Complete...");
	return 1;
}

public OnGameModeExit()
{
	return 1;
}

public OnPlayerRequestClass(playerid, classid)
{
	SetPlayerTeamFromClass(playerid, classid);
	SetupPlayerForClassSelection(playerid);
 	new teamName[23];
	switch (classid) {
		case 0,1, 2:
		    {
				format(teamName,23, "~p~%s", gTeamInfo[TEAM_2][TeamName]);
				GameTextForPlayer(playerid, teamName, 1000, 3);
			}
	    case 3, 4, 5:
	        {
				format(teamName,23, "~g~%s", gTeamInfo[TEAM_1][TeamName]);
				GameTextForPlayer(playerid, teamName, 1000, 3);
			}
	}
	return 1;
}

public TeamBalanceCheck()
{
	new team1, team2, team1_lastid, team2_lastid;
	for(i=0;i<GetMaxPlayers();i++) {
	    if(gPlayer[i][Team] == TEAM_1) {
	        team1++;
	        team1_lastid = i;
     	}
     	else if (gPlayer[i][Team] == TEAM_2) {
     	    team2++;
     	    team2_lastid = i;
     	}
	}
	if(team1 == team2) {
		//when there are no players connected, reset score.
		if (team1==0) {
			for(i=1;i<sizeof(gTeamInfo);i++) {
				if (gTeamInfo[i][TeamScore] != 0) {
				    printf("[cartheft] No players left, resetting score to 0 for team %d", i);
					gTeamInfo[i][TeamScore] = 0;
					AddTeamScore(i,0);
				}
			}
		}
		return;
	}
	//printf("[debug] result of teambalance check: TEAM1: %d, TEAM2: %d", team1, team2);
	//only act on teamdifference of 3 or higher
	if ((team1 - team2) > 2) {
	    //team1 is at least 3 players larger, switch player
		ChangePlayerTeam(team1_lastid);
	}
	else if ((team2 - team1) > 2) {
	    ChangePlayerTeam(team2_lastid);
	}
	return;
}

ChangePlayerTeam(playerid)
{
	if(gPlayer[playerid][Team] == TEAM_1)
	{
		gPlayer[playerid][Team] = TEAM_2;
		gPlayer[playerid][Skin] = gPlayer[playerid][Skin] - 3;
	}
	else if(gPlayer[playerid][Team] == TEAM_2)
	{
		gPlayer[playerid][Team] = TEAM_1;
		gPlayer[playerid][Skin] = gPlayer[playerid][Skin] + 3;
	}
	else {
		printf("[ERROR] invalid team in ChangePlayerTeam, player: %d, team: %d", playerid, gPlayer[playerid][Team]);
		Kick(playerid);
		return 0;
	}
	SetPlayerTeam(playerid, gPlayer[playerid][Team]);
	LockCarsForPlayer(playerid);
	SetPlayerColor(playerid, gTeamInfo[gPlayer[playerid][Team]][TeamColor]);
    ResetSpawnInfo(playerid);
    GivePlayerMoneyLimit(playerid, PENALTY_DEATH); //give money to die for
	SetPlayerHealth(playerid, 0); //kill the unfortunate player :(
    
    new string[64];
	format(string, 64, "[teambalance] %s was switched to team %s", ReturnPlayerName(playerid), gTeamInfo[gPlayer[playerid][Team]][TeamName]);
	SendClientMessageToAll(COLOR_GREY, string);
	printf(string);
	return 1;
}

SetPlayerTeamFromClass(playerid, classid)
{
	gPlayer[playerid][Skin] = 102 + classid;
	if (classid < 3)
	{
		gPlayer[playerid][Team] = TEAM_2;
	}
	else
	{
		gPlayer[playerid][Team] = TEAM_1;
	}
	SetPlayerTeam(playerid, gPlayer[playerid][Team]);
}

LockCarsForPlayer(playerid) {
	// printf("[debug] player: %d classid = %d team = %s",playerid, classid, gTeamInfo[gPlayer[playerid][Team]][TeamName]);
	for (i=1;i<sizeof(gCars);i++) {
	    if(gCars[i][CarTeam] == gPlayer[playerid][Team]) {
			//Lock Cars from same team
			//printf("[debug] locking car %d for player %d", i, playerid);
			SetVehicleParamsForPlayer(i,playerid,0,1);
	    }
	    else {
	        //Unlock Cars from other team
	        //printf("[debug] unlocking car %d for player %d", i, playerid);
	        SetVehicleParamsForPlayer(i,playerid,0,0);
	    }
	}
}


// Move player to changingroom
SetupPlayerForClassSelection(playerid)
{
	SetPlayerInterior(playerid,14);
	SetPlayerPos(playerid,258.4893,-41.4008,1002.0234);
	SetPlayerFacingAngle(playerid, 90.0);
	SetPlayerCameraPos(playerid,256.0815,-43.0475,1003.0234);
	SetPlayerCameraLookAt(playerid,258.4893,-41.4008,1002.0234);
    SetPlayerColor(playerid,COLOR_GREY);
	return;
}

AddTeamScore(teamid, points) {
	if (gTeamWon) return 1; //don't add more points if a team has already won
	gTeamInfo[teamid][TeamScore] = gTeamInfo[teamid][TeamScore] + points;
	new string[64];
	if(gTeamInfo[TEAM_1][TeamScore] < gTeamInfo[TEAM_2][TeamScore]) {
		format(string, 64, "%s:~w~ %d ~n~%s:~w~ %d",
			gTeamInfo[TEAM_2][TeamScoreName],
			gTeamInfo[TEAM_2][TeamScore],
			gTeamInfo[TEAM_1][TeamScoreName],
			gTeamInfo[TEAM_1][TeamScore]);
    }
    else {
		format(string, 64, "%s:~w~ %d ~n~%s:~w~ %d",
			gTeamInfo[TEAM_1][TeamScoreName],
			gTeamInfo[TEAM_1][TeamScore],
			gTeamInfo[TEAM_2][TeamScoreName],
			gTeamInfo[TEAM_2][TeamScore]);
    }
    TextDrawSetString(gScore, string);
    if(gTeamInfo[teamid][TeamScore] >= SCORE_WINLIMIT) {
        printf("[cartheft] Team %s reached winlimit", gTeamInfo[teamid][TeamName]);
		gTeamWon = teamid;
		format(string, 64, "Team %s Wins!", gTeamInfo[teamid][TeamName]);
		GameTextForAll(string,10000,6);
		SetTimer("EndGame", 10000, 0);
		for(i=0;i<GetMaxPlayers();i++) {
			if(gPlayer[i][Team] == TEAM_NEUTRAL) {
		    }
		    else if (gPlayer[i][Team] == teamid) {
				//ApplyAnimation(i,"RIOT","RIOT_CHANT",4.1,0,1,1,1,1);
		        //PlayerPlaySound(i,SOUND_CROWD_CHEERS_BIG,0.0,0.0,0.0);
				PlayerPlaySound(i,SOUND_WIN,0.0,0.0,0.0);
				//remove this line if the sound is not music.
				gPlayer[i][Sound] = SOUND_WIN;
				//add a little extra points to the player
				SetPlayerScore(i,GetPlayerScore(i)+5);
		    }
			else {
				//ApplyAnimation(i,"GANGS","Invite_No",4.1,0,1,1,1,1);
                //PlayerPlaySound(i,SOUND_CROWD_AWWS,0.0,0.0,0.0);
				PlayerPlaySound(i,SOUND_LOOSE ,0.0,0.0,0.0);
				//remove this line if the sound is not music.
				gPlayer[i][Sound] = SOUND_LOOSE;
			}
		}
    }
    return 1;
}

public EndGame() {
	TextDrawDestroy(gScore);
	SetTimer("ExitGameMode", 1500, 0);
}

public ExitGameMode() {
	GameModeExit();
}

public OnPlayerRequestSpawn(playerid)
{
	SetPlayerTeam(playerid, gPlayer[playerid][Team]);
	LockCarsForPlayer(playerid);
	SetPlayerColor(playerid, gTeamInfo[gPlayer[playerid][Team]][TeamColor]);
	ResetSpawnInfo(playerid);
	return 1;
}

public OnPlayerConnect(playerid)
{
   	TextDrawShowForPlayer(playerid, gScore);
	SetPlayerInterior(playerid,14);
	SetPlayerPos(playerid,258.4893,-41.4008,1002.0234);
	GameTextForPlayer(playerid,"~w~SA:MP Car Theft!",4000,3);
    SetPlayerColor(playerid,COLOR_GREY);
 	SetPlayerMapIcon(playerid, 0, ICON_TEAM1, gTeamInfo[TEAM_1][TeamIcon], 0);
 	SetPlayerMapIcon(playerid, 1, ICON_TEAM2, gTeamInfo[TEAM_2][TeamIcon], 0);
 	SendClientMessage(playerid,COLOR_ORANGE, "Welcome to Car Theft. Type /help for help");
	return 1;
}

public OnPlayerDisconnect(playerid, reason)
{
	//Loop through all cars to check if the stolenby flag was set for this player
	for (i=1;i<sizeof(gCars);i++) {
		if (gCars[i][StolenByPlayerId] == playerid) {
			gCars[i][StolenByPlayerId] = -1;
		}
	}
//	ClearAnimations(playerid);
//	SetPlayerSpecialAction(playerid, SPECIAL_ACTION_NONE);
	//stop playing sound (end music is always current music +1)
	PlayerPlaySound(playerid,gPlayer[playerid][Sound]+1,0.0,0.0,0.0);
	gPlayer[playerid][Sound] = -1;
	//Use this to display correct team colors on the scoreboard.
	SetPlayerColor(playerid, gTeamInfo[gPlayer[playerid][Team]][TeamColor]);
	TextDrawHideForPlayer(playerid, gScore);
	return 1;
}

public OnPlayerSpawn(playerid)
{
	SetPlayerWorldBounds(playerid, WORLD_BOUNDS);
	SetPlayerTeam(playerid, gPlayer[playerid][Team]);
	SetPlayerColor(playerid, gTeamInfo[gPlayer[playerid][Team]][TeamColor]);
	GameTextForPlayer(playerid, "Let's go steal some cars!", 2000, 5);
	TogglePlayerControllable(playerid, true);
	ResetSpawnInfo(playerid);
	return 1;
}

ResetSpawnInfo(playerid)
{
    SetPlayerInterior(playerid, 0);
	switch (gPlayer[playerid][Team]) {
	    case TEAM_1:
    	{
			new rand = random(sizeof(gTeamGreenSpawns));
			SetSpawnInfo(playerid, TEAM_1, gPlayer[playerid][Skin],
				gTeamGreenSpawns[rand][PlayerX],
				gTeamGreenSpawns[rand][PlayerY],
				gTeamGreenSpawns[rand][PlayerZ],
				gTeamGreenSpawns[rand][PlayerAngle],
				0, 0, 0, 0, 0, 0);
		}
 	    case TEAM_2:
    	{
			new rand = random(sizeof(gTeamPurpleSpawns));
			SetSpawnInfo(playerid, TEAM_2, gPlayer[playerid][Skin],
				gTeamPurpleSpawns[rand][PlayerX],
				gTeamPurpleSpawns[rand][PlayerY],
				gTeamPurpleSpawns[rand][PlayerZ],
				gTeamPurpleSpawns[rand][PlayerAngle],
				0, 0, 0, 0, 0, 0);
		}
	}
}

CheatCheck(playerid) {
	new Float:armour;
 	GetPlayerArmour(playerid,armour);
	if (IsWeaponRestricted(GetPlayerWeapon(playerid)) || armour > 0 || GetPlayerMoney(playerid) > ANTICHEAT_MAX_MONEY) {
		new string[64];
		format(string,64,"Cheater (scripted) W: %d, A:%d, M: %d", GetPlayerWeapon(playerid), floatround(armour),  GetPlayerMoney(playerid));
		BanEx(playerid, string);
	}
	return 1;
}

GivePlayerMoneyLimit(playerid, amount) {
	if (GetPlayerMoney(playerid) + amount > ANTICHEAT_MAX_MONEY) {
		ResetPlayerMoney(playerid);
		GivePlayerMoney(playerid, ANTICHEAT_MAX_MONEY);
	}
	else if (GetPlayerMoney(playerid) + amount < 0) {
		ResetPlayerMoney(playerid);
	}
	else {
		GivePlayerMoney(playerid, amount);
	}
}

public OnPlayerDeath(playerid, killerid, reason)
{
	DisablePlayerCheckpoint(playerid);
	GivePlayerMoneyLimit(playerid, PENALTY_DEATH * -1);
	if(IsPlayerConnected(killerid) && killerid != 255) {
		CheatCheck(killerid);
		//printf("[debug] player %s(%d) team %d was killed by player %s(%d) team %d with %d",ReturnPlayerName(playerid), playerid, gPlayer[playerid][Team],ReturnPlayerName(killerid), killerid, gPlayer[killerid][Team], reason );
		if(gPlayer[playerid][Team] == TEAM_1) {
	        if (IsPlayerInArea(playerid, HOMEAREA_TEAM2) && gPlayer[killerid][Team] == TEAM_2) {
				//Player was in homearea award killer
				new string[64];
				format(string, 64, "%s defended the %s garage by killing %s",ReturnPlayerName(killerid), gTeamInfo[TEAM_2][TeamName], ReturnPlayerName(playerid));
				SendClientMessageToAll(gTeamInfo[TEAM_2][TeamColor], string);
				GivePlayerMoneyLimit(killerid, AWARD_HOMEDEFENCE);
				SetPlayerScore(killerid,GetPlayerScore(killerid)+1);
				GameTextForPlayer(killerid, "Nice defending!", 1000, 5);
				AddTeamScore(TEAM_2, SCORE_DEFEND_BASE);
			}
		}
		else if (gPlayer[playerid][Team] == TEAM_2) {
	        if (IsPlayerInArea(playerid, HOMEAREA_TEAM1) && gPlayer[killerid][Team] == TEAM_1 ) {
				//Player was in homearea award killer
				new string[64];
				format(string, 64, "%s defended the %s garage by killing %s",ReturnPlayerName(killerid), gTeamInfo[TEAM_1][TeamName], ReturnPlayerName(playerid));
				SendClientMessageToAll(gTeamInfo[TEAM_1][TeamColor], string);
				GivePlayerMoneyLimit(killerid, AWARD_HOMEDEFENCE);
				SetPlayerScore(killerid,GetPlayerScore(killerid)+1);
				GameTextForPlayer(killerid, "Nice defending!", 1000, 5);
				AddTeamScore(TEAM_1, SCORE_DEFEND_BASE);
	        }
		}
		
		for(i=1;i<sizeof(gCars);i++) {
			if(gCars[i][StolenByPlayerId] == playerid) {
				//Player was stealing a car Award Killer
				new string[64];
				format(string, 64, "%s defended the %s by killing %s",ReturnPlayerName(killerid), gCars[i][CarName], ReturnPlayerName(playerid));
				SendClientMessageToAll(COLOR_GREY, string);
				GivePlayerMoneyLimit(killerid, AWARD_CARDEFENCE);
				GameTextForPlayer(killerid, "Nice defending!", 1000, 5);
				AddTeamScore(gPlayer[killerid][Team], SCORE_DEFEND_CAR);
		    }
		}
	}
	AddTeamScore(gPlayer[killerid][Team], SCORE_KILL);
	return 1;
}

public OnVehicleSpawn(vehicleid)
{
	//Destroy non-default (bought) vehicles
	if( vehicleid > gStaticVehicles) DestroyVehicle(vehicleid);
	if( vehicleid > sizeof(gCars)) return 1;
	//Reset "stolen by"
	gCars[vehicleid][StolenByPlayerId] = -1;
	return 1;
}

public OnVehicleDeath(vehicleid, killerid)
{
	if(vehicleid > sizeof(gCars)) return 1;
	if(killerid == 255 && gCars[vehicleid][StolenByPlayerId] != -1) {
		killerid = gCars[vehicleid][StolenByPlayerId];
	}
	else {
	    return 1;
	}
	if (gCars[vehicleid][CarTeam] == TEAM_NEUTRAL) {
		return 1;
	}
	new price = (gCars[vehicleid][CarCostPrice]);
	new string[64];
	format(string, 64, "The %s is destroyed! this costs %d$!", gCars[vehicleid][CarName], price);
	if (GetPlayerMoney(killerid) < price) {
		price = GetPlayerMoney(killerid);
		format(string, 64, "The %s is destroyed! this costs %d$ and a score point!", gCars[vehicleid][CarName], price);
		SetPlayerScore(killerid,GetPlayerScore(killerid)-1);
		if (price == 0) {
			format(string, 64, "The %s is destroyed! this costs your life and a score point!", gCars[vehicleid][CarName], price);
		}
	}
	GivePlayerMoneyLimit(killerid, price * -1);
	GameTextForPlayer(killerid, string, 6000, 1);
	if (price == 0) SetPlayerHealth(killerid, 0);
	format(string, 64, "%s wrecked the %s",ReturnPlayerName(killerid), gCars[vehicleid][CarName]);
	SendClientMessageToAll(COLOR_GREY, string);
	AddTeamScore(gCars[vehicleid][CarTeam], SCORE_OTHERS_WRECKED_CAR);
	printf("[cartheft] %s", string);
	return 1;
}

public OnPlayerText(playerid, text[])
{
	return 1;
}

public OnPlayerPrivmsg(playerid, recieverid, text[])
{
	return 1;
}

public OnPlayerCommandText(playerid, cmdtext[])
{
	if (strcmp("/buy", cmdtext, true, 4) == 0)
	{
		ShowMenuForPlayer(buymenu, playerid);
		TogglePlayerControllable(playerid, false);
		CheatCheck(playerid);
		return 1;
	}
	if ((strcmp("/objective", cmdtext, true) == 0) || (strcmp("/help", cmdtext, true) == 0)) {
   		SendClientMessage(playerid,COLOR_ORANGE,"====) CAR THEFT (====");
		SendClientMessage(playerid,COLOR_LIGHTBLUE,"A Teamplay car stealing gamemode.");
   		SendClientMessage(playerid,COLOR_LIGHTBLUE,"Steal Cars from the other team and bring them to your own garage.");
   		SendClientMessage(playerid,COLOR_LIGHTBLUE,"Be carefull, damaged cars have less value!");
   		SendClientMessage(playerid,COLOR_LIGHTBLUE,"The garages are marked on your map.");
		new string[64];
		format(string, 64, "The first team to %d points wins!", SCORE_WINLIMIT);
   		SendClientMessage(playerid,COLOR_LIGHTBLUE, string);
   		SendClientMessage(playerid,COLOR_LIGHTBLUE,"type /buy to buy defence, cars or weapons");
   		SendClientMessage(playerid,COLOR_LIGHTBLUE,"type /kill to respawn");
   		SendClientMessage(playerid,COLOR_ORANGE,"====) CAR THEFT (====");
   		return 1;
	}
	if (strcmp("/kill", cmdtext, true) == 0)
	{
		new price = PENALTY_SUICIDE;
		if (GetPlayerMoney(playerid) < price) {
			price = GetPlayerMoney(playerid);
		}
		GivePlayerMoneyLimit(playerid, price * -1);
		SetPlayerHealth(playerid, 0);
		return 1;
	}
	//Admin commands
	if (IsPlayerAdmin(playerid)) {
		if (strcmp("/money",cmdtext,true)==0) {
			printf("[admin] %s got 100000$", ReturnPlayerName(playerid));
			GivePlayerMoneyLimit(playerid, 100000);
			return 1;
		}
		if (strcmp("/switchteam", cmdtext, true, 11)==0) {
			printf("[admin] changeteam player %d", strval(cmdtext[12]));
			ChangePlayerTeam(strval(cmdtext[12]));
		    return 1;
		}
		if (strcmp("/restart", cmdtext, true, 8)==0) {
			printf("[admin] endgame");
			EndGame();
			return 1;
		}
	}
	return 0;
}

public OnPlayerInfoChange(playerid)
{
	return 1;
}

public OnPlayerEnterVehicle(playerid, vehicleid, ispassenger)
{
	return 1;
}

GetVehicleValue(vehicleid) {
	new Float:health, Float:fPrice;
	new price;
	GetVehicleHealth(vehicleid, health);
	fPrice = gCars[vehicleid][CarSellPrice] * health * 0.00001;
	if (health < 0.200) {
	    fPrice = 10;
	}
	else if (health < 0.500) {
	    fPrice = fPrice * 0.3;
	}
	else if (health < 0.800) {
		fPrice = fPrice * 0.5;
	}
	else if (health < 0.900) {
	    fPrice = fPrice * 0.7;
	}
	price = floatround(fPrice);
    if (price < 10) price = 10;
	return price * 100;
}

public OnPlayerExitVehicle(playerid, vehicleid)
{
	DisablePlayerCheckpoint(playerid);
	return 1;
}

public OnPlayerStateChange(playerid, newstate, oldstate)
{
    if(newstate == PLAYER_STATE_DRIVER)
    {
		new vehicleid = GetPlayerVehicleID(playerid);
		if (vehicleid > sizeof(gCars)) return 1;
		if (gCars[vehicleid][CarTeam] == gPlayer[playerid][Team]) return 1;
		//Don't display message again if the same player enters the car again.
		if(!(gCars[vehicleid][StolenByPlayerId] == playerid)) {
			gCars[vehicleid][StolenByPlayerId] = playerid;
			printf("[cartheft] %s stole the %s!", ReturnPlayerName(playerid), gCars[vehicleid][CarName]);
			new string[64];
			format(string, 64, "You took the %s\n current value: %d$!", gCars[vehicleid][CarName], GetVehicleValue(vehicleid));
			GameTextForPlayer(playerid, string, 3000, 4);
			format(string, 64, "%s is stealing the %s", ReturnPlayerName(playerid), gCars[vehicleid][CarName]);
			SendClientMessageToAll(COLOR_GREY, string);
		}
		if(gPlayer[playerid][Team] == TEAM_1) {
			SetPlayerCheckpoint(playerid, CHECK_TEAM1, 5.0);
		}
		else if (gPlayer[playerid][Team] == TEAM_2) {
			SetPlayerCheckpoint(playerid, CHECK_TEAM2, 5.0);
		}
		else { //unknown team, weird, kick player
			printf("[ERROR] Player %d had no team and changed to driver", playerid);
		    Kick( playerid );
		}
	}
	return 1;
}

public OnPlayerEnterCheckpoint(playerid)
{
	if (!IsPlayerInAnyVehicle(playerid)) return 1;
	new vehicleid = GetPlayerVehicleID(playerid);
	if ( vehicleid != 0 && vehicleid < sizeof(gCars) ) {
		CheatCheck(playerid);
		printf("[cartheft] %s completed stealing the %s", ReturnPlayerName(playerid), gCars[vehicleid][CarName]);
		GivePlayerMoneyLimit(playerid, GetVehicleValue(vehicleid));
		SetPlayerScore(playerid,GetPlayerScore(playerid)+1);
		SetVehicleToRespawn(vehicleid);
		DisablePlayerCheckpoint(playerid);
		new string[64];
		format(string, 64, "This %s is worth %d$!", gCars[vehicleid][CarName], GetVehicleValue(vehicleid));
		GameTextForPlayer(playerid, string, 5000, 6);
		format(string, 64, "%s stole the %s worth still %d$", ReturnPlayerName(playerid), gCars[vehicleid][CarName], GetVehicleValue(vehicleid));
		SendClientMessageToAll(COLOR_GREY, string);
		AddTeamScore(gPlayer[playerid][Team], SCORE_STEAL_CAR);
	}
	return 1;
}

public OnPlayerLeaveCheckpoint(playerid)
{
	return 1;
}

public OnPlayerEnterRaceCheckpoint(playerid)
{
	return 1;
}

public OnPlayerLeaveRaceCheckpoint(playerid)
{
	return 1;
}

public OnRconCommand(cmd[])
{
	return 1;
}

public OnObjectMoved(objectid)
{
	return 1;
}

public OnPlayerObjectMoved(playerid, objectid)
{
	return 1;
}

public OnPlayerPickUpPickup(playerid, pickupid)
{
	return 1;
}

public OnPlayerSelectedMenuRow(playerid, row)
{
	TogglePlayerControllable(playerid, true);
	new Menu:CurrentMenu = GetPlayerMenu(playerid);
	if(CurrentMenu == buymenu)
	{
		if (GetPlayerMoney(playerid) < gBuyMenuItems[row][Price]) {
		    SendClientMessage(playerid,COLOR_ORANGE,"You don't have enough money");
		    return 1;
		}
  		else if(gBuyMenuItems[row][Type] == TYPE_OBJECT) {
			if (IsPlayerInAnyVehicle(playerid)){
		    	SendClientMessage(playerid,COLOR_ORANGE,"Cant buy from within a vehicle");
			}
			else {
				GivePlayerMoneyLimit(playerid, gBuyMenuItems[row][Price] * -1); //pay
	  			new Float:x,Float:y,Float:z,Float:a;
				GetPlayerPos(playerid,x,y,z);
				GetPlayerFacingAngle(playerid, a);
				// Calculate object location, 1 point in front of the player
				x = floatsub(x, floatsin(a, degrees)); //remove 1 times the sine of the angle
				y = floatadd(y, floatcos(a, degrees)); //add 1 times the cosine of the angle
				z = floatsub(z, 0.45); //lower it 0.45
				CreateObject(1225, x, y, z, 0,0,0);
			}
  		}
		else if(gBuyMenuItems[row][Type] == TYPE_WEAPON) {
			GivePlayerMoneyLimit(playerid, gBuyMenuItems[row][Price] * -1); // pay
			GivePlayerWeapon(playerid,gBuyMenuItems[row][Id], gBuyMenuItems[row][Ammo]);
			new string[64];
			format(string, 64, "You bought a %s for %s",gBuyMenuItems[row][Name], gBuyMenuItems[row][PriceString]);
            SendClientMessage(playerid,COLOR_ORANGE,string);
		}
		else if(gBuyMenuItems[row][Type] == TYPE_VEHICLE) {
			if (IsPlayerInAnyVehicle(playerid)){
		    	SendClientMessage(playerid,COLOR_ORANGE,"Cant buy from within a vehicle");
			}
			else {
			    GivePlayerMoneyLimit(playerid, gBuyMenuItems[row][Price] * -1);
			    new Float:x, Float:y,Float:z,Float:angle;
			    GetPlayerPos(playerid,x,y,z);
			    GetPlayerFacingAngle(playerid, angle);
    			PutPlayerInVehicle(playerid, CreateVehicle(gBuyMenuItems[row][Id], x, y, z, angle, -1, -1, gRespawnDelay), 0);
				new string[64];
				format(string, 64, "You bought yourself a new %s for %s",gBuyMenuItems[row][Name], gBuyMenuItems[row][PriceString]);
	            SendClientMessage(playerid,COLOR_ORANGE,string);
    		}
  		}
		else {
			SendClientMessage(playerid,COLOR_ORANGE,"Unknown Type");
		}
	}
	return 1;
}

public OnPlayerExitedMenu(playerid)
{
	TogglePlayerControllable(playerid, true);
	return 1;
}

IsWeaponRestricted(weaponid) {
	switch(weaponid) {
		case RESTRICTED_WEAPONS: return true;
		default: return false;
	}
	return false;
}

IsPlayerInArea(playerid, Float:minx, Float:maxx, Float:miny, Float:maxy) //By Alex "Y_Less" Cole
{
	new Float:x, Float:y, Float:z;
	GetPlayerPos(playerid, x, y, z);
	if (x > minx && x < maxx && y > miny && y < maxy) return 1;
	return 0;
}

ReturnPlayerName(playerid)
{
	new pname[MAX_PLAYER_NAME];
	GetPlayerName(playerid, pname, sizeof (pname));
	return pname;
}
